#ifndef __COPAMH_H__
#define __COPAMH_H__ 

#include "copula.h"

/*
 * CLASS for the implementation of a Ali-Mikhail-Haq
 * Bivariate Copula
 */

class CopAmh : public Copula
{
 private:

  double rho;       /* the (single) theta parameter */
  double rho_win;   /* the RW proposal paramter for rho */

  Copula* RWrho(double *lqfwd, double *lqbak) const;
  Copula* RWrho(double start, double *lqfwd) const;
  double  RWrhoEval(double start) const;

  void initialize();

 protected:

  virtual double evalSlice(unsigned int which, double value);

 public:

  CopAmh(double *d);
  CopAmh(double rho, const CopAmh *old);
  virtual ~CopAmh(void);
  virtual void Cuti(unsigned int n, double *u, double *v) const;
  virtual double LLik(unsigned int n, double *u, double *v) const;
  virtual double LLik(unsigned int n, double *u, double *v, unsigned int *idx) const;
  virtual double LLik(double *u, double *v, set<int> *idx) const;
  virtual double LLik(double u, double v) const;
  virtual unsigned int NumParams(void) const;
  virtual void Params(double *out) const;
  virtual Copula* RW(unsigned int which, double *lqfwd, double *lqbak) const;
  virtual Copula* RW(double start, unsigned int which, double *lqfwd) const;
  virtual double evalRW(unsigned int which, double value) const;
  virtual double LPrior(void) const;
  virtual void   randomize();
  virtual double randomizeLL() const;
  virtual void   setParams(double *in);
  virtual double getParam(unsigned int which) const;

  virtual double spearmanRho();
  virtual void   getImpliedSpearmanParams(double srho, double out[]);

  virtual Copula* cloneCopula(unsigned int which, double new_param) const;
  virtual double  priorSample() const;

  double *AMH_SPEARMAN;
  double *AMH_SPEARMAN_B;
  int    AMH_SPEARMAN_SIZE;
};

#endif
