#ifndef __COPNORM_H__
#define __COPNORM_H__ 

#include "copula.h"

/*
 * CLASS for the implementation of a Normal 
 * Bivariate Copula
 */

class CopNorm : public Copula
{
 private:

  double rho;       /* the (single) theta parameter */
  double rho_win;   /* the RW proposal paramter for rho */

  Copula* RWrho(double *lqfwd, double *lqbak) const;
  Copula* RWrho(double start, double *lqfwd) const;
  double  RWrhoEval(double value) const;
 
 protected:

  virtual double evalSlice(unsigned int which, double value);

 public:

  CopNorm(double *d);
  CopNorm(double rho, const CopNorm *old);
  virtual ~CopNorm(void);
  virtual void Cuti(unsigned int n, double *u, double *v) const;

  virtual double LLik(unsigned int n, double *u, double *v) const;
  virtual double LLik(unsigned int n, double *u, double *v, unsigned int *idx) const;
  virtual double LLik(double *u, double *v, set<int> *idx) const;
  virtual double LLik(double u, double v) const;

  virtual unsigned int NumParams(void) const;
  virtual void Params(double *out) const;
  virtual Copula* RW(unsigned int which, double *lqfwd, double *lqbak) const;
  virtual Copula* RW(double start, unsigned int which, double *lqfwd) const;
  virtual double evalRW(unsigned int which, double value) const;
  virtual double LPrior(void) const;
  virtual void   randomize();
  virtual double randomizeLL() const;
  virtual void   setParams(double *in);
  virtual double getParam(unsigned int which) const;

  virtual double spearmanRho();
  virtual void   getImpliedSpearmanParams(double srho, double out[]);

  virtual Copula* cloneCopula(unsigned int which, double new_param) const;
  virtual double  priorSample() const;
};

#endif
