
#include "factory.h"

#include "copamh.h"
#include "copclayton.h"
#include "copfgm.h"
#include "copfrank.h"
#include "copgb.h"
#include "copnorm.h"

#include "margempirical.h"
#include "margexp.h"
#include "marggamma.h"
#include "margnormal.h"
#include "margstudent.h"

/* 
 * new_marg: 
 * 
 * return an allocated marginal module based on the 
 * type indicated in the first entry of, where the rest
 * of the entries contain parameters, initialized with
 * the data passed in.
 */

Marginal* new_marg(unsigned int n, double *x, double *d)
{
  unsigned int type = (unsigned int) d[0];
  switch(type) {
     case 1: return new Normal(n, x, d + 1);
     case 2: return new Student(n, x, d + 1);
     case 3: return new Gamma(n, x, d + 1);  
     case 4: return new Exponential(n, x, d + 1);  
     case 5: return new Empirical(n, x, d + 1);  
     default: error("marginal type %d not known\n", type);
  }
  return NULL;
}

/*
 * new_copula:
 *
 * allocate the proper copula module as required by R -- 
 * marginals must be allocated first
 */

Copula* new_copula(double *d)
{
  unsigned int type = (unsigned int) d[0];
  switch(type) {
      case 1: return new CopNorm(d + 1);
      case 2: return new CopFgm(d + 1);
      case 3: return new CopAmh(d + 1);
      case 4: return new CopClayton(d + 1);
      case 5: return new CopGb(d + 1);
      case 6: return new CopFrank(d + 1);
      default: error("copula type %d not known\n", type);
  }
  return NULL;
}

