#ifndef __MARGEXPONENTIAL_H__
#define __MARGEXPONENTIAL_H__ 

#include "marginal.h"
#include <fstream>

/*
 * CLASS for the implementation of a Exponential marginal
 * distribution in the context of inferring bivariate
 * distribution defined by a pair of marginals and a copula
 */

class Exponential : public Marginal
{
 private:

  double a;        /* mean parameter */
  double a_win;    /* RW proposal variance for the mean */
  double a_a, a_b; /* Hyperparameters for a */

  Marginal* RWa(double *lqfwd, double *lqbak) const;

 public:

  Exponential(double a);
  Exponential(unsigned int n, double *x, double *d);
  Exponential(double a, double a_a, double a_b, const Exponential *old);  

  virtual ~Exponential(void);
  virtual double* NewRanks(unsigned int n, double *z);
  virtual void CalcRanks(void);
  virtual double CalcRanks(double x) const;
  virtual double LLik(void) const;
  virtual double LLik(double x) const;
  virtual unsigned int NumParams(void) const;
  virtual void Params(double *out) const;
  virtual Marginal* RW(unsigned int which, double *lqfwd, double *lqbak) const;
  virtual double RWvalue(double value, double *lqfwd, double *lqbak) const;
  virtual double LPrior(void) const;
  virtual void setParams(double *in);
  virtual void randomize();
};

#endif
