#ifndef __MARGGAMMA_H__
#define __MARGGAMMA_H__ 

#include "marginal.h"
#include <fstream>

/*
 * CLASS for the implementation of a Gamma marginal
 * distribution in the context of inferring bivariate
 * distribution defined by a pair of marginals and a copula
 */

class Gamma : public Marginal
{
 private:

  double a;        /* mean parameter */
  double a_win;    /* RW proposal variance for the mean */
  double a_a, a_b; /* Hyperparameters for a */
  double b;        /* variance parameter */
  double b_win;   /* RW proposal for parameer s2 */
  double b_a, b_b; /* Hyperparameters for a */

  Marginal* RWa(double *lqfwd, double *lqbak) const;
  Marginal* RWb(double *lqfwd, double *lqbak) const;

 public:

  Gamma(double a, double b);
  Gamma(unsigned int n, double *x, double *d);
  Gamma(double a, double b, double a_a, double a_b, double b_a, double b_b, const Gamma *old);  

  virtual ~Gamma(void);
  virtual double* NewRanks(unsigned int n, double *z);
  virtual void CalcRanks(void);
  virtual double CalcRanks(double x) const;
  virtual double LLik(void) const;
  virtual double LLik(double x) const;
  virtual unsigned int NumParams(void) const;
  virtual void Params(double *out) const;
  virtual Marginal* RW(unsigned int which, double *lqfwd, double *lqbak) const;
  virtual double RWvalue(double value, double *lqfwd, double *lqbak) const;
  virtual double LPrior(void) const;
  virtual void setParams(double *in);
  virtual void randomize();
};

#endif
