#ifndef __MARGNORMAL_H__
#define __MARGNORMAL_H__ 

#include "marginal.h"
#include <fstream>

/*
 * CLASS for the implementation of a Normal marginal
 * distribution in the context of inferring bivariate
 * distribution defined by a pair of marginals and a copula
 */

class Normal : public Marginal
{
 private:

  double mu;        /* mean parameter */
  double mu_s2q;    /* RW proposal variance for the mean */
  double s2;        /* variance parameter */
  double s2_prop;   /* RW proposal for parameer s2 */

  Marginal* RWmu(double *lqfwd, double *lqbak) const;
  Marginal* RWs2(double *lqfwd, double *lqbak) const;

 public:

  Normal(double mu, double s2);
  Normal(unsigned int n, double *x, double *d);
  Normal(double mu, double s2, const Normal *old);

  virtual ~Normal(void);
  virtual double* NewRanks(unsigned int n, double *z);
  virtual void CalcRanks(void);
  virtual double CalcRanks(double x) const;
  virtual double LLik(void) const;
  virtual double LLik(double x) const;
  virtual unsigned int NumParams(void) const;
  virtual void Params(double *out) const;
  virtual Marginal* RW(unsigned int which, double *lqfwd, double *lqbak) const;
  virtual double RWvalue(double value, double *lqfwd, double *lqbak) const;
  virtual double LPrior(void) const;
  virtual void setParams(double *in);
  virtual void randomize();
};

#endif
