#ifndef __MARGSTUDENT_H__
#define __MARGSTUDENT_H__ 

#include "marginal.h"
#include <fstream>

/*
 * CLASS for the implementation of a Student-t marginal
 * distribution in the context of inferring bivariate
 * distribution defined by a pair of marginals and a copula
 */

class Student : public Marginal
{
 private:

  double mu;         /* mean parameter */
  //double mu_m, mu_v; /* hyperparameters mean and variance of the Gaussian prior of the mean */
  double mu_s2q;     /* RW proposal variance for the mean */

  double s2;         /* variance parameter */
  //double s2_a, s2_b; /* hyperparameters of the inverse gamma used as priors for the variance */
  double s2_prop;    /* RW proposal parameter for s2 */

  double nu;         /* degrees of freedom parameter */
  double nu_prop;    /* RW proposal parameter for nu */

  Marginal* RWmu(double *lqfwd, double *lqbak) const;
  Marginal* RWs2(double *lqfwd, double *lqbak) const;
  Marginal* RWnu(double *lqfwd, double *lqbak) const;

 public:

  Student(double mu, double s2, double nu);
  Student(unsigned int n, double *x, double *d);
  Student(double mu, double s2, double nu, const Student *old);
  virtual ~Student(void);
  virtual double* NewRanks(unsigned int n, double *z);
  virtual void CalcRanks(void);
  virtual double CalcRanks(double x) const;
  virtual double LLik(void) const;
  virtual double LLik(double x) const;
  virtual unsigned int NumParams(void) const;
  virtual void Params(double *out) const;
  virtual Marginal* RW(unsigned int which, double *lqfwd, double *lqbak) const;
  virtual double RWvalue(double value, double *lqfwd, double *lqbak) const;
  virtual double LPrior(void) const;
  virtual void setParams(double *in);
  virtual void randomize();
};

#endif
